/* Turbo C Driver for FTP Software's packet driver interface.
 * Graciously donated to the public domain by Phil Karn.
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <dos.h>

#include "pktdrvr.h"

#ifdef BC2
#define _dos_getvect	getvect
#endif

#define MY_OFF(p)	(((unsigned short far *)&(p))[0])
#define MY_SEG(p)	(((unsigned short far *)&(p))[1])

extern int	Derr = 0;
extern void far pkset (int portno, void (far *receiver)(), int far *stk, 
	int dseg);

/* Test for the presence of a packet driver at an interrupt number.
 * Return 0 if no packet driver.
*/

#define	DRVR_SIG	"PKT DRVR"

#define	intdrvr(i,rr,sr)	int86x (i, &rr, &rr, &sr)

extern int far
test_for_pd (unsigned int intno)
{
	union {
		void	(INTERRUPT far *addr)();
		char	far *name;
	}	drvvec;

	drvvec.addr = _dos_getvect (intno);
	return (!memcmp (DRVR_SIG, drvvec.name+3, sizeof (DRVR_SIG)));
}

extern int far
driver_info (int intno, int handle, int *version, int *class, int *type,
	int *number, char **name, int *basic)
{
	union REGS regs;
	struct SREGS sregs;

	segread (&sregs);
	regs.x.bx = handle;
	regs.h.ah = DRIVER_INFO;
	regs.h.al = 0xff;
	intdrvr (intno, regs, sregs);
	if (regs.x.cflag) {
		Derr = regs.h.dh;
		return -1;
	}
	if (version != NULL)
		*version = regs.x.bx;
	if (class != NULL)
		*class = regs.h.ch;
	if (type != NULL)
		*type = regs.x.dx;
	if (number != NULL)
		*number = regs.h.cl;
	if (name != NULL) {
		MY_SEG (*name) = sregs.ds;
		MY_OFF (*name) = regs.x.si;
	}
	if (basic != NULL)
		*basic = regs.h.al;
	return 0;
}

static char	far * near dseg = (char far *)&dseg;

extern void far
pkinit (int portno, void (far *receiver)(), int far *stack)
{
	pkset (portno, receiver, stack, MY_SEG (dseg));
}

extern int far
access_type (int intno, int if_class, int if_type, int if_number,
	char far *type, unsigned typelen,
	void (INTERRUPT *receiver) __ARGS((void)))
{
	union REGS regs;
	struct SREGS sregs;

	segread (&sregs);
	regs.h.dl = (char)if_number;	/* Number */
	sregs.ds = MY_SEG(type);	/* Packet type template */
	regs.x.si = MY_OFF(type);
	regs.x.cx = typelen;		/* Length of type */
	sregs.es = MY_SEG(receiver);	/* Address of receive handler */
	regs.x.di = MY_OFF(receiver);
	regs.x.bx = if_type;		/* Type */
	regs.h.ah = ACCESS_TYPE;	/* Access_type() function */
	regs.h.al = (char)if_class;	/* Class */
	intdrvr (intno, regs, sregs);
	if (regs.x.cflag) {
		Derr = regs.h.dh;
		return -1;
	} else
		return regs.x.ax;
}

extern int far
release_type (int intno, int handle)
{
	union REGS regs;
	struct SREGS sregs;

	segread (&sregs);
	regs.x.bx = handle;
	regs.h.ah = RELEASE_TYPE;
	intdrvr (intno, regs, sregs);
	if (regs.x.cflag) {
		Derr = regs.h.dh;
		return -1;
	} else
		return 0;
}

extern int far
send_pkt (int intno, char far *buffer, unsigned length)
{
	union REGS regs;
	struct SREGS sregs;

	segread (&sregs);
	sregs.ds  = MY_SEG(buffer);
	sregs.es  = MY_SEG(buffer); /* for buggy univation pkt driver - CDY */
	regs.x.si = MY_OFF(buffer);
	regs.x.cx = length;
	regs.h.ah = SEND_PKT;
	intdrvr (intno, regs, sregs);
	if (regs.x.cflag) {
		Derr = regs.h.dh;
		return -1;
	} else
		return 0;
}

extern int far
drvr_terminate (int intno, int handle)
{
	union REGS	regs;
	struct SREGS	sregs;

	segread (&sregs);
	regs.x.bx = handle;
	regs.h.ah = TERMINATE;
	intdrvr (intno, regs, sregs);
	if (regs.x.cflag) {
		Derr = regs.h.dh;
		return -1;
	}
	return 0;
}

extern int far
get_address (int intno, int handle, char *buf, int len)
{
	union REGS	regs;
	struct SREGS	sregs;

	segread (&sregs);
	sregs.es  = MY_SEG (buf);
	regs.x.di = MY_OFF (buf);
	regs.x.cx = len;
	regs.x.bx = handle;
	regs.h.ah = GET_ADDRESS;
	intdrvr (intno, regs, sregs);
	if (regs.x.cflag) {
		Derr = regs.h.dh;
		return -1;
	}
	return 0;
}

extern int far
reset_interface (int intno, int handle)
{
	union REGS	regs;
	struct SREGS	sregs;

	segread (&sregs);
	regs.x.bx = handle;
	regs.h.ah = RESET_INTERFACE;
	intdrvr (intno, regs, sregs);
	if (regs.x.cflag) {
		Derr = regs.h.dh;
		return -1;
	}
	return 0;
}

extern int far
get_parameters(int intno, struct param **params)
{
	union REGS	regs;
	struct SREGS	sregs;

	if (!params)
		return (-2);

	segread (&sregs);
	regs.h.ah = GET_PARAMETERS;
	intdrvr (intno, regs, sregs);
	if (regs.x.cflag) {
		Derr = regs.h.dh;
		return -1;
	}
	MY_SEG (*params) = sregs.es;
	MY_OFF (*params) = regs.x.di;
	return 0;
}

extern int far
set_address (int intno, char *addr, int len)
{
	union REGS	regs;
	struct SREGS	sregs;

	segread (&sregs);
	sregs.es  = MY_SEG (addr);
	regs.x.di = MY_OFF (addr);
	regs.x.cx = len;
	regs.h.ah = SET_ADDRESS;
	intdrvr (intno, regs, sregs);
	if (regs.x.cflag) {
		Derr = regs.h.dh;
		return -1;
	}
	return 0;
}

extern int far
get_statistics (int intno, int handle, struct statistics **addr)
{
	union REGS	regs;
	struct SREGS	sregs;

	if (!addr)
		return -2;

	segread (&sregs);
	regs.x.bx = handle;
	regs.h.ah = GET_STATISTICS;
	intdrvr (intno, regs, sregs);
	if (regs.x.cflag) {
		Derr = regs.h.dh;
		return -1;
	}
	MY_SEG(*addr) = sregs.es;
	MY_OFF(*addr) = regs.x.di;
	return 0;
}

extern int far
as_send_pkt (int intno, char far *buffer, unsigned length,
	void (INTERRUPT *upcall) __ARGS((void)))
{
	union REGS regs;
	struct SREGS sregs;

	segread (&sregs);
	sregs.ds  = MY_SEG(buffer);
	regs.x.si = MY_OFF(buffer);
	regs.x.cx = length;
	sregs.es  = MY_SEG(upcall);
	regs.x.di = MY_OFF(upcall);
	regs.h.ah = AS_SEND_PKT;
	intdrvr (intno, regs, sregs);
	if (regs.x.cflag) {
		Derr = regs.h.dh;
		return -1;
	} else
		return 0;
}
